@@ -24,6 +24,7 @@ modules = [
     "adultempire",
     "agnph",
     "ao3",
+    "arcalive",
     "architizer",
     "artstation",
     "aryion",
@@ -44,6 +45,7 @@ modules = [
     "danbooru",
     "desktopography",
     "deviantart",
+    "discord",
     "dynastyscans",
     "e621",
     "erome",
@@ -56,6 +58,7 @@ modules = [
     "fapachi",
     "flickr",
     "furaffinity",
+    "furry34",
     "fuskator",
     "gelbooru",
     "gelbooru_v01",
@@ -65,7 +68,6 @@ modules = [
     "hentai2read",
     "hentaicosplays",
     "hentaifoundry",
-    "hentaifox",
     "hentaihand",
     "hentaihere",
     "hentainexus",
@@ -80,6 +82,7 @@ modules = [
     "imgbox",
     "imgth",
     "imgur",
+    "imhentai",
     "inkbunny",
     "instagram",
     "issuu",
@@ -168,6 +171,8 @@ modules = [
     "tapas",
     "tcbscans",
     "telegraph",
+    "tenor",
+    "tiktok",
     "tmohentai",
     "toyhouse",
     "tsumino",
