@@ -175,6 +175,51 @@ class DanbooruExtractor(BaseExtractor):
         return [{"file": fmt(index), "delay": delay}
                 for index, delay in enumerate(delays)]
 
+    def _collection_posts(self, cid, ctype):
+        reverse = prefix = None
+
+        order = self.config("order-posts")
+        if not order or order in {"asc", "pool", "pool_asc", "asc_pool"}:
+            params = {"tags": "ord{}:{}".format(ctype, cid)}
+        elif order in {"id", "desc_id", "id_desc"}:
+            params = {"tags": "{}:{}".format(ctype, cid)}
+            prefix = "b"
+        elif order in {"desc", "desc_pool", "pool_desc"}:
+            params = {"tags": "ord{}:{}".format(ctype, cid)}
+            reverse = True
+        elif order in {"asc_id", "id_asc"}:
+            params = {"tags": "{}:{}".format(ctype, cid)}
+            reverse = True
+
+        posts = self._pagination("/posts.json", params, prefix)
+        if reverse:
+            self.log.info("Collecting posts of %s %s", ctype, cid)
+            return self._collection_enumerate_reverse(posts)
+        else:
+            return self._collection_enumerate(posts)
+
+    def _collection_metadata(self, cid, ctype, cname=None):
+        url = "{}/{}s/{}.json".format(self.root, cname or ctype, cid)
+        collection = self.request(url).json()
+        collection["name"] = collection["name"].replace("_", " ")
+        self.post_ids = collection.pop("post_ids", ())
+        return {ctype: collection}
+
+    def _collection_enumerate(self, posts):
+        pid_to_num = {pid: num for num, pid in enumerate(self.post_ids, 1)}
+        for post in posts:
+            post["num"] = pid_to_num[post["id"]]
+            yield post
+
+    def _collection_enumerate_reverse(self, posts):
+        posts = list(posts)
+        posts.reverse()
+
+        pid_to_num = {pid: num for num, pid in enumerate(self.post_ids, 1)}
+        for post in posts:
+            post["num"] = pid_to_num[post["id"]]
+        return posts
+
 
 BASE_PATTERN = DanbooruExtractor.update({
     "danbooru": {
@@ -205,12 +250,8 @@ class DanbooruTagExtractor(DanbooruExtractor):
     pattern = BASE_PATTERN + r"/posts\?(?:[^&#]*&)*tags=([^&#]*)"
     example = "https://danbooru.donmai.us/posts?tags=TAG"
 
-    def __init__(self, match):
-        DanbooruExtractor.__init__(self, match)
-        tags = match.group(match.lastindex)
-        self.tags = text.unquote(tags.replace("+", " "))
-
     def metadata(self):
+        self.tags = text.unquote(self.groups[-1].replace("+", " "))
         return {"search_tags": self.tags}
 
     def posts(self):
@@ -232,27 +273,37 @@ class DanbooruTagExtractor(DanbooruExtractor):
 
 
 class DanbooruPoolExtractor(DanbooruExtractor):
-    """Extractor for posts from danbooru pools"""
+    """Extractor for Danbooru pools"""
     subcategory = "pool"
     directory_fmt = ("{category}", "pool", "{pool[id]} {pool[name]}")
+    filename_fmt = "{num:>04}_{id}_{filename}.{extension}"
     archive_fmt = "p_{pool[id]}_{id}"
     pattern = BASE_PATTERN + r"/pool(?:s|/show)/(\d+)"
     example = "https://danbooru.donmai.us/pools/12345"
 
-    def __init__(self, match):
-        DanbooruExtractor.__init__(self, match)
-        self.pool_id = match.group(match.lastindex)
-
     def metadata(self):
-        url = "{}/pools/{}.json".format(self.root, self.pool_id)
-        pool = self.request(url).json()
-        pool["name"] = pool["name"].replace("_", " ")
-        self.post_ids = pool.pop("post_ids", ())
-        return {"pool": pool}
+        return self._collection_metadata(self.groups[-1], "pool")
 
     def posts(self):
-        params = {"tags": "pool:" + self.pool_id}
-        return self._pagination("/posts.json", params, "b")
+        return self._collection_posts(self.groups[-1], "pool")
+
+
+class DanbooruFavgroupExtractor(DanbooruExtractor):
+    """Extractor for Danbooru favorite groups"""
+    subcategory = "favgroup"
+    directory_fmt = ("{category}", "Favorite Groups",
+                     "{favgroup[id]} {favgroup[name]}")
+    filename_fmt = "{num:>04}_{id}_{filename}.{extension}"
+    archive_fmt = "fg_{favgroup[id]}_{id}"
+    pattern = BASE_PATTERN + r"/favorite_group(?:s|/show)/(\d+)"
+    example = "https://danbooru.donmai.us/favorite_groups/12345"
+
+    def metadata(self):
+        return self._collection_metadata(
+            self.groups[-1], "favgroup", "favorite_group")
+
+    def posts(self):
+        return self._collection_posts(self.groups[-1], "favgroup")
 
 
 class DanbooruPostExtractor(DanbooruExtractor):
@@ -262,12 +313,8 @@ class DanbooruPostExtractor(DanbooruExtractor):
     pattern = BASE_PATTERN + r"/post(?:s|/show)/(\d+)"
     example = "https://danbooru.donmai.us/posts/12345"
 
-    def __init__(self, match):
-        DanbooruExtractor.__init__(self, match)
-        self.post_id = match.group(match.lastindex)
-
     def posts(self):
-        url = "{}/posts/{}.json".format(self.root, self.post_id)
+        url = "{}/posts/{}.json".format(self.root, self.groups[-1])
         post = self.request(url).json()
         if self.includes:
             params = {"only": self.includes}
@@ -283,12 +330,8 @@ class DanbooruPopularExtractor(DanbooruExtractor):
     pattern = BASE_PATTERN + r"/(?:explore/posts/)?popular(?:\?([^#]*))?"
     example = "https://danbooru.donmai.us/explore/posts/popular"
 
-    def __init__(self, match):
-        DanbooruExtractor.__init__(self, match)
-        self.params = match.group(match.lastindex)
-
     def metadata(self):
-        self.params = params = text.parse_query(self.params)
+        self.params = params = text.parse_query(self.groups[-1])
         scale = params.get("scale", "day")
         date = params.get("date") or datetime.date.today().isoformat()
 
