@@ -52,16 +52,18 @@ class ToyhouseExtractor(Extractor):
         return {
             "url": extr(needle, '"'),
             "date": text.parse_datetime(extr(
-                'Credits\n</h2>\n<div class="mb-1">', '<'),
+                '</h2>\n            <div class="mb-1">', '<'),
                 "%d %b %Y, %I:%M:%S %p"),
             "artists": [
                 text.remove_html(artist)
                 for artist in extr(
-                    '<div class="artist-credit">', '</div>\n</div>').split(
-                    '<div class="artist-credit">')
+                    '<div class="artist-credit">',
+                    '</div>\n                    </div>').split(
+                    '<div class="ar tist-credit">')
             ],
             "characters": text.split_html(extr(
-                '<div class="image-characters', '</div>\n</div>'))[2:],
+                '<div class="image-characters',
+                '<div class="image-comments">'))[2:],
         }
 
     def _pagination(self, path):
@@ -77,23 +79,27 @@ class ToyhouseExtractor(Extractor):
                 cnt += 1
                 yield self._parse_post(post)
 
-            if cnt == 0 and params["page"] == 1:
-                token, pos = text.extract(
-                    page, '<input name="_token" type="hidden" value="', '"')
-                if not token:
-                    return
-                data = {
-                    "_token": token,
-                    "user"  : text.extract(page, 'value="', '"', pos)[0],
-                }
-                self.request(self.root + "/~account/warnings/accept",
-                             method="POST", data=data, allow_redirects=False)
-                continue
+            if not cnt and params["page"] == 1:
+                if self._accept_content_warning(page):
+                    continue
+                return
 
             if cnt < 18:
                 return
             params["page"] += 1
 
+    def _accept_content_warning(self, page):
+        pos = page.find(' name="_token"') + 1
+        token, pos = text.extract(page, ' value="', '"', pos)
+        user , pos = text.extract(page, ' value="', '"', pos)
+        if not token or not user:
+            return False
+
+        data = {"_token": token, "user": user}
+        self.request(self.root + "/~account/warnings/accept",
+                     method="POST", data=data, allow_redirects=False)
+        return True
+
 
 class ToyhouseArtExtractor(ToyhouseExtractor):
     """Extractor for artworks of a toyhouse user"""
@@ -119,4 +125,5 @@ class ToyhouseImageExtractor(ToyhouseExtractor):
 
     def posts(self):
         url = "{}/~images/{}".format(self.root, self.user)
-        return (self._parse_post(self.request(url).text, '<img src="'),)
+        return (self._parse_post(
+            self.request(url).text, '<img class="mw-100" src="'),)
