@@ -10,6 +10,7 @@
 
 from .common import Extractor, Message
 from .. import text, exception
+import re
 
 BASE_PATTERN = r"(?:https://)?(?:www\.|m\.)?vk\.com"
 
@@ -23,7 +24,15 @@ class VkExtractor(Extractor):
     root = "https://vk.com"
     request_interval = (0.5, 1.5)
 
+    def _init(self):
+        self.offset = text.parse_int(self.config("offset"))
+
+    def skip(self, num):
+        self.offset += num
+        return num
+
     def items(self):
+        sub = re.compile(r"/imp[fg]/").sub
         sizes = "wzyxrqpo"
 
         data = self.metadata()
@@ -40,11 +49,15 @@ class VkExtractor(Extractor):
                 continue
 
             try:
-                photo["url"] = photo[size + "src"]
+                url = photo[size + "src"]
             except KeyError:
                 self.log.warning("no photo URL found (%s)", photo.get("id"))
                 continue
 
+            photo["url"] = sub("/", url.partition("?")[0])
+            #  photo["url"] = url
+            photo["_fallback"] = (url,)
+
             try:
                 _, photo["width"], photo["height"] = photo[size]
             except ValueError:
@@ -69,7 +82,7 @@ class VkExtractor(Extractor):
             "al"       : "1",
             "direction": "1",
             "list"     : photos_id,
-            "offset"   : 0,
+            "offset"   : self.offset,
         }
 
         while True:
