@@ -55,8 +55,28 @@ class MetadataPP(PostProcessor):
             self._json_encode = self._make_encoder(options, 4).encode
             ext = "json"
 
+        base_directory = options.get("base-directory")
+        if base_directory:
+            if base_directory is True:
+                self._base = lambda p: p.basedirectory
+            else:
+                sep = os.sep
+                altsep = os.altsep
+                base_directory = util.expand_path(base_directory)
+                if altsep and altsep in base_directory:
+                    base_directory = base_directory.replace(altsep, sep)
+                if base_directory[-1] != sep:
+                    base_directory += sep
+                self._base = lambda p: base_directory
+
         directory = options.get("directory")
-        if directory:
+        if isinstance(directory, list):
+            self._directory = self._directory_format
+            self._directory_formatters = [
+                formatter.parse(dirfmt, util.NONE).format_map
+                for dirfmt in directory
+            ]
+        elif directory:
             self._directory = self._directory_custom
             sep = os.sep + (os.altsep or "")
             self._metadir = util.expand_path(directory).rstrip(sep) + os.sep
@@ -83,10 +103,10 @@ class MetadataPP(PostProcessor):
         job.register_hooks({event: self.run for event in events}, options)
 
         self._init_archive(job, options, "_MD_")
+        self.filter = self._make_filter(options)
         self.mtime = options.get("mtime")
         self.omode = options.get("open", omode)
         self.encoding = options.get("encoding", "utf-8")
-        self.private = options.get("private", False)
         self.skip = options.get("skip", False)
 
     def run(self, pathfmt):
@@ -94,7 +114,10 @@ class MetadataPP(PostProcessor):
         if archive and archive.check(pathfmt.kwdict):
             return
 
-        directory = self._directory(pathfmt)
+        if util.WINDOWS and pathfmt.extended:
+            directory = pathfmt._extended_path(self._directory(pathfmt))
+        else:
+            directory = self._directory(pathfmt)
         path = directory + self._filename(pathfmt)
 
         if self.skip and os.path.exists(path):
@@ -141,11 +164,27 @@ class MetadataPP(PostProcessor):
             except Exception:
                 pass
 
-    def _directory(self, pathfmt):
+    def _base(self, pathfmt):
         return pathfmt.realdirectory
 
+    def _directory(self, pathfmt):
+        return self._base(pathfmt)
+
     def _directory_custom(self, pathfmt):
-        return os.path.join(pathfmt.realdirectory, self._metadir)
+        return os.path.join(self._base(pathfmt), self._metadir)
+
+    def _directory_format(self, pathfmt):
+        formatters = pathfmt.directory_formatters
+        conditions = pathfmt.directory_conditions
+        try:
+            pathfmt.directory_formatters = self._directory_formatters
+            pathfmt.directory_conditions = ()
+            segments = pathfmt.build_directory(pathfmt.kwdict)
+            directory = pathfmt.clean_path(os.sep.join(segments) + os.sep)
+            return os.path.join(self._base(pathfmt), directory)
+        finally:
+            pathfmt.directory_conditions = conditions
+            pathfmt.directory_formatters = formatters
 
     def _filename(self, pathfmt):
         return (pathfmt.filename or "metadata") + "." + self.extension
@@ -195,10 +234,33 @@ class MetadataPP(PostProcessor):
         fp.write("\n".join(tags) + "\n")
 
     def _write_json(self, fp, kwdict):
-        if not self.private:
-            kwdict = util.filter_dict(kwdict)
+        if self.filter:
+            kwdict = self.filter(kwdict)
         fp.write(self._json_encode(kwdict) + "\n")
 
+    def _make_filter(self, options):
+        include = options.get("include")
+        if include:
+            if isinstance(include, str):
+                include = include.split(",")
+            return lambda d: {k: d[k] for k in include if k in d}
+
+        exclude = options.get("exclude")
+        private = options.get("private")
+        if exclude:
+            if isinstance(exclude, str):
+                exclude = exclude.split(",")
+            exclude = set(exclude)
+
+            if private:
+                return lambda d: {k: v for k, v in d.items()
+                                  if k not in exclude}
+            return lambda d: {k: v for k, v in util.filter_dict(d).items()
+                              if k not in exclude}
+
+        if not private:
+            return util.filter_dict
+
     @staticmethod
     def _make_encoder(options, indent=None):
         return json.JSONEncoder(
