@@ -48,6 +48,19 @@ def bdecode(data, alphabet="0123456789"):
     return num
 
 
+def decrypt_xor(encrypted, key, base64=True, fromhex=False):
+    if base64:
+        encrypted = binascii.a2b_base64(encrypted)
+    if fromhex:
+        encrypted = bytes.fromhex(encrypted.decode())
+
+    div = len(key)
+    return bytes([
+        encrypted[i] ^ key[i % div]
+        for i in range(len(encrypted))
+    ]).decode()
+
+
 def advance(iterable, num):
     """"Advance 'iterable' by 'num' steps"""
     iterator = iter(iterable)
@@ -356,6 +369,31 @@ def extract_headers(response):
     return data
 
 
+def detect_challenge(response):
+    server = response.headers.get("server")
+    if not server:
+        return
+
+    elif server.startswith("cloudflare"):
+        if response.status_code not in (403, 503):
+            return
+
+        mitigated = response.headers.get("cf-mitigated")
+        if mitigated and mitigated.lower() == "challenge":
+            return "Cloudflare challenge"
+
+        content = response.content
+        if b"_cf_chl_opt" in content or b"jschl-answer" in content:
+            return "Cloudflare challenge"
+        elif b'name="captcha-bypass"' in content:
+            return "Cloudflare CAPTCHA"
+
+    elif server.startswith("ddos-guard"):
+        if response.status_code == 403 and \
+                b"/ddos-guard/js-challenge/" in response.content:
+            return "DDoS-Guard challenge"
+
+
 @functools.lru_cache(maxsize=None)
 def git_head():
     try:
@@ -648,8 +686,10 @@ class CustomNone():
 
 
 # v128.0 release on 2024-07-09 has ordinal 739076
+# v137.0 release on 2025-04-01 has ordinal 739342
 # 735492 == 739076 - 128 * 28
-_ff_ver = (datetime.date.today().toordinal() - 735492) // 28
+# 735506 == 739342 - 137 * 28
+_ff_ver = (datetime.date.today().toordinal() - 735506) // 28
 
 NONE = CustomNone()
 EPOCH = datetime.datetime(1970, 1, 1)
@@ -770,7 +810,7 @@ def import_file(path):
         finally:
             del sys.path[0]
     else:
-        return __import__(name)
+        return __import__(name.replace("-", "_"))
 
 
 def build_duration_func(duration, min=0.0):
